/* GTK - The GIMP Toolkit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * GtkDisplay: Emulate a simple DISPLAY (light emitting diode)
 * Copyright (C) 1997 Tim Janik
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <gnome.h>
#include <stdio.h>
#include "gtkdisplay.h"

static void gtk_display_class_init	 (GtkDisplayClass	 *klass);
static void gtk_display_init	 (GtkDisplay	 *display);
static void gtk_display_destroy	 (GtkObject	 *object);
#if 0
static void gtk_display_size_request (GtkWidget	 *widget,
				  GtkRequisition *requisition);
static void gtk_display_size_allocate (GtkWidget         *widget,
          GtkAllocation     *allocation);
#endif
static gint gtk_display_expose	 (GtkWidget	 *widget,
				  GdkEventExpose *event);
static void gtk_display_realize      (GtkWidget      *widget);


static GtkVBoxClass *parent_class = NULL;

enum {
  DISPLAY_COLOR_ON,
  DISPLAY_COLOR_OFF,
  DISPLAY_COLOR_OVER
};

GtkType
gtk_display_get_type ()
{
  static GtkType display_type = 0;
  
  if (!display_type)
  {
    GtkTypeInfo display_info =
    {
      "GtkDisplay",
      sizeof (GtkDisplay),
      sizeof (GtkDisplayClass),
      (GtkClassInitFunc) gtk_display_class_init,
      (GtkObjectInitFunc) gtk_display_init,
      /* reserved */ NULL,
      /* reserved */ NULL,
      (GtkClassInitFunc) NULL,
    };
    
    display_type = gtk_type_unique (gtk_vbox_get_type (), &display_info);
  }
  
  return display_type;
}

void
gtk_display_class_init (GtkDisplayClass *class)
{
  GtkObjectClass *object_class;
  GtkWidgetClass *widget_class;
  
  object_class = (GtkObjectClass*) class;
  widget_class = (GtkWidgetClass*) class;
  
  parent_class = gtk_type_class (gtk_vbox_get_type ());
  
  object_class->destroy = gtk_display_destroy;
  
//  widget_class->size_request  = gtk_display_size_request;
//  widget_class->size_allocate  = gtk_display_size_allocate;
  widget_class->expose_event  = gtk_display_expose;
  widget_class->realize       = gtk_display_realize;
}

void
gtk_display_init (GtkDisplay *display)
{
  int x;
  
  GTK_WIDGET_SET_FLAGS (display, GTK_NO_WINDOW);
  
  display->value             = 0;
  display->digits            = DIGITS;
  display->gc                = NULL;
	for(x = 0; x < MAX_DIGITS; x++)
		display->led[x] = NULL;
}

static void create_widget(GtkDisplay *display) {
	int x;

	gdk_color_parse("#FF1710", &(display->fg[DISPLAY_COLOR_ON]));
	gdk_color_parse("#400000", &(display->fg[DISPLAY_COLOR_OFF]));
	gdk_color_parse("#17FF10", &(display->fg[DISPLAY_COLOR_OVER]));

	display->table = gtk_table_new(display->digits, 1, FALSE);
	gtk_container_add(GTK_CONTAINER(display), display->table);

	for(x = 0; x < display->digits; x++)
	{
		display->led[x] = gtk_led_new();
		gtk_table_attach_defaults(GTK_TABLE(display->table),
					  display->led[x],
					  x, x+1, 0, 1);
		gtk_led_set_colors(GTK_LED(display->led[x]), &(display->fg[DISPLAY_COLOR_ON]),
		       &(display->fg[DISPLAY_COLOR_OFF]));
		gtk_widget_show(display->led[x]);
	}

	gtk_widget_show_all(display->table);
}

GtkWidget*
gtk_display_new_with_digits (guint digits)
{
	GtkDisplay *display;
  
	display = gtk_type_new (gtk_display_get_type ());
	display->digits = digits;

	create_widget(display);

	return GTK_WIDGET (display);
}

GtkWidget*
gtk_display_new ()
{
	return(gtk_display_new_with_digits(DIGITS));  
}

void
gtk_display_set_colors (GtkDisplay *display, GdkColor *active, GdkColor *inactive, GdkColor *over)
{
	int x;

	g_return_if_fail (display != NULL);
	g_return_if_fail (GTK_IS_DISPLAY (display));

	for(x = 0; x < display->digits; x++)
	{
		gtk_led_set_colors(GTK_LED(display->led[x]), active, inactive);
		gtk_widget_show(display->led[x]);
	}

	display->fg[DISPLAY_COLOR_ON] = *(active);
	display->fg[DISPLAY_COLOR_OFF] = *(inactive);
	display->fg[DISPLAY_COLOR_OVER] = *(over);
}

void
gtk_display_set_state (GtkDisplay	*display,
		   GtkStateType widget_state,
		   guint	value)
{
  g_return_if_fail (display != NULL);
  g_return_if_fail (GTK_IS_DISPLAY (display));

  gtk_widget_set_state (GTK_WIDGET (display), widget_state);
  gtk_display_set_value (display, value);
}

void
gtk_display_set_digits (GtkDisplay	     *display,
		guint     value)
{
	int z;
	int count;

	g_return_if_fail (display != NULL);
	g_return_if_fail (GTK_IS_DISPLAY (display));

	if(value == display->digits)
		return;

	if(value > MAX_DIGITS)
		value = MAX_DIGITS;

	/* destroy the old display */
	for(z = 0; z < display->digits; z++) {
		if(display->led[z]) {
			gtk_widget_destroy(display->led[z]);
			display->led[z] = NULL;
		}
	}
	if(display->table)
		gtk_widget_destroy(display->table);
		
	/* rebuild */
	display->digits = value;

	create_widget(display);

	count = display->value;
  	for(z = (display->digits-1); z >= 0; z--) {
		gtk_led_set_value(GTK_LED(display->led[z]), count%10);
		count = count / 10;
	}
}

void
gtk_display_set_value (GtkDisplay	     *display,
		guint     value)
{
	int z;
	int count;
  g_return_if_fail (display != NULL);
  g_return_if_fail (GTK_IS_DISPLAY (display));

  display->value = value;
	count = value;
  	for(z = (display->digits-1); z >= 0; z--) {
		gtk_led_set_value(GTK_LED(display->led[z]), count%10);
		count = count / 10;
	}

 	for(z = (display->digits-1); z >= 0; z--) {
		if(count > 0)
			gtk_led_set_colors(GTK_LED(display->led[z]),
			     &(display->fg[DISPLAY_COLOR_OVER]),
		       &(display->fg[DISPLAY_COLOR_OFF]));
		else
			gtk_led_set_colors(GTK_LED(display->led[z]),
			     &(display->fg[DISPLAY_COLOR_ON]),
		       &(display->fg[DISPLAY_COLOR_OFF]));
	}

  gtk_widget_draw (GTK_WIDGET (display), NULL);
}

guint
gtk_display_get_value(GtkDisplay  *display)
{
  g_return_val_if_fail (display != NULL, -1);
  g_return_val_if_fail (GTK_IS_DISPLAY (display), -1);
  
  return display->value;
}

static void
gtk_display_destroy (GtkObject *object)
{
  GtkDisplay *display;
  
  g_return_if_fail (object != NULL);
  g_return_if_fail (GTK_IS_DISPLAY (object));
  
  display = GTK_DISPLAY (object);
  
  if (GTK_WIDGET (object)->parent &&
      GTK_WIDGET_MAPPED (object))
    gtk_widget_unmap (GTK_WIDGET (object));
  
  if (GTK_OBJECT_CLASS (parent_class)->destroy)
    (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
}

#if 0
static void
gtk_display_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
{
	GtkDisplay *display;
  
	g_return_if_fail (widget != NULL);
	g_return_if_fail (GTK_IS_DISPLAY (widget));
	g_return_if_fail (allocation != NULL);

//	fprintf(stderr, "display: size_allocate called: %d, %d\n", allocation->width, allocation->height);
  
	display = GTK_DISPLAY (widget);

	widget->allocation = *allocation;
}

static void
gtk_display_size_request (GtkWidget	     *widget,
		      GtkRequisition *requisition)
{
	GtkDisplay *display;
  
	g_return_if_fail (widget != NULL);
	g_return_if_fail (GTK_IS_DISPLAY (widget));
	g_return_if_fail (requisition != NULL);
  
	display = GTK_DISPLAY (widget);
  
//fprintf(stderr, "gtkdisplay: height = %d, width = %d\n", requisition->height,  requisition->width);

	requisition->width = -1; //LED_WIDTH * display->digits; // + display->misc.xpad * 2;
	requisition->height = -1; //DISPLAY_HEIGHT; // + display->misc.ypad * 2 + BOTTOM_SPACE;
}
#endif

static void 
gtk_display_realize (GtkWidget *widget)
{
	GtkDisplay       *display;

	g_return_if_fail (widget != NULL);
	g_return_if_fail (GTK_IS_DISPLAY (widget));

	GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
	display = GTK_DISPLAY (widget);
	widget->window = gtk_widget_get_parent_window (widget);
	gdk_window_ref (widget->window);
	widget->style = gtk_style_attach (widget->style, widget->window);
}

static gint
gtk_display_expose (GtkWidget      *widget,
		GdkEventExpose *event)
{
  GtkDisplay   *display;
  GtkVBox  *misc;
  gint     i;

  g_return_val_if_fail (widget != NULL, FALSE);
  g_return_val_if_fail (GTK_IS_DISPLAY (widget), FALSE);
  g_return_val_if_fail (event != NULL, FALSE);
  
  display = GTK_DISPLAY (widget);
  misc = GTK_VBOX (widget);

	for(i = 0; i < display->digits; i++)
	{
		gtk_widget_draw(GTK_WIDGET (display->led[i]), NULL);
	}

  return TRUE;
}
/* EOF */

