/* GTK - The GIMP Toolkit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * GtkLed: Emulate a simple LED (light emitting diode)
 * Copyright (C) 1997 Tim Janik
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
//#include <config.h>
#include <stdio.h>
#include "gtkled.h"

#ifndef MIN
#define MIN(a,b) (((a)<(b))?(a):(b))
#endif

static void gtk_led_class_init	 (GtkLedClass	 *klass);
static void gtk_led_init	 (GtkLed	 *led);
static void gtk_led_destroy	 (GtkObject	 *object);
#if 0
static void gtk_led_size_request (GtkWidget	 *widget,
				  GtkRequisition *requisition);
#endif
static void gtk_led_size_allocate (GtkWidget	 *widget,
				  GtkAllocation *allocation);
static void gtk_led_realize      (GtkWidget      *widget);
static void gtk_led_draw (GtkWidget         *widget, GdkRectangle      *area);


static GtkMiscClass *parent_class = NULL;

enum {
  LED_COLOR_ON,
  LED_COLOR_OFF
};


GtkType
gtk_led_get_type ()
{
  static GtkType led_type = 0;
  
  if (!led_type)
  {
    GtkTypeInfo led_info =
    {
      "GtkLed",
      sizeof (GtkLed),
      sizeof (GtkLedClass),
      (GtkClassInitFunc) gtk_led_class_init,
      (GtkObjectInitFunc) gtk_led_init,
      /* reserved */ NULL,
      /* reserved */ NULL,
      (GtkClassInitFunc) NULL,
    };
    
    led_type = gtk_type_unique (gtk_misc_get_type (), &led_info);
  }
  
  return led_type;
}

void
gtk_led_class_init (GtkLedClass *class)
{
  GtkObjectClass *object_class;
  GtkWidgetClass *widget_class;
  
  object_class = (GtkObjectClass*) class;
  widget_class = (GtkWidgetClass*) class;
  
  parent_class = gtk_type_class (gtk_misc_get_type ());
  
  object_class->destroy = gtk_led_destroy;
  
//  widget_class->size_request  = gtk_led_size_request;
  widget_class->size_allocate = gtk_led_size_allocate;
  widget_class->realize       = gtk_led_realize;
  widget_class->draw          = gtk_led_draw;
}

void
gtk_led_init (GtkLed *led)
{
  GtkMisc *misc;
  
  misc = GTK_MISC (led);

  GTK_WIDGET_SET_FLAGS (led, GTK_NO_WINDOW);
  
  led->value             = 0;
  led->gc                = NULL;

}

GtkWidget*
gtk_led_new ()
{
  GtkLed *led;
  
  led = gtk_type_new (gtk_led_get_type ());

  return GTK_WIDGET (led);
}

void
gtk_led_set_colors (GtkLed *led, GdkColor *active, GdkColor *inactive)
{
	GdkColormap  *cmap;

	g_return_if_fail (led != NULL);
	g_return_if_fail (GTK_IS_LED (led));

	led->fg[LED_COLOR_OFF] = *(inactive);
	led->fg[LED_COLOR_ON] = *(active);

	cmap = gtk_widget_get_colormap (GTK_WIDGET(led));
      
	if (!(&(led->fg[LED_COLOR_ON])))
		gdk_color_parse ("#00F100", &(led->fg[LED_COLOR_ON]));
	gdk_color_alloc (cmap, &(led->fg[LED_COLOR_ON]));
	if (!(&(led->fg[LED_COLOR_OFF])))
		gdk_color_parse ("#008C00", &(led->fg[LED_COLOR_OFF]));
	gdk_color_alloc (cmap, &(led->fg[LED_COLOR_OFF]));
}

void
gtk_led_set_state (GtkLed	*led,
		   GtkStateType widget_state,
		   guint	value)
{
  g_return_if_fail (led != NULL);
  g_return_if_fail (GTK_IS_LED (led));

  gtk_widget_set_state (GTK_WIDGET (led), widget_state);
  gtk_led_set_value (led, value);
}

void
gtk_led_set_value (GtkLed	     *led,
		guint     value)
{
  g_return_if_fail (led != NULL);
  g_return_if_fail (GTK_IS_LED (led));

  led->value = value;
  gtk_widget_draw (GTK_WIDGET (led), NULL);
}

guint
gtk_led_get_value(GtkLed  *led)
{
  g_return_val_if_fail (led != NULL, -1);
  g_return_val_if_fail (GTK_IS_LED (led), -1);
  
  return led->value;
}

static void
gtk_led_destroy (GtkObject *object)
{
  GtkLed *led;
  
  g_return_if_fail (object != NULL);
  g_return_if_fail (GTK_IS_LED (object));
  
  led = GTK_LED (object);
  
  if (GTK_WIDGET (object)->parent &&
      GTK_WIDGET_MAPPED (object))
    gtk_widget_unmap (GTK_WIDGET (object));
  
  if (GTK_OBJECT_CLASS (parent_class)->destroy)
    (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
}

static void
gtk_led_size_allocate (GtkWidget	     *widget,
		      GtkAllocation *allocation)
{
  GtkLed *led;
  
  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_LED (widget));
  g_return_if_fail (allocation != NULL);
  
//	fprintf(stderr, "led: size_allocate called: %d, %d\n", allocation->width, allocation->height);
  led = GTK_LED (widget);
  
	widget->allocation = *allocation;
}

#if 0
static void
gtk_led_size_request (GtkWidget	     *widget,
		      GtkRequisition *requisition)
{
  GtkLed *led;
  
  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_LED (widget));
  g_return_if_fail (requisition != NULL);
  
  led = GTK_LED (widget);
  
  requisition->width = LED_WIDTH;
  requisition->height = LED_HEIGHT;
}
#endif

static void 
gtk_led_realize (GtkWidget *widget)
{
  GtkLed       *led;
  GdkColormap  *cmap;

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_LED (widget));

  GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
  led = GTK_LED (widget);
  widget->window = gtk_widget_get_parent_window (widget);
  gdk_window_ref (widget->window);
  widget->style = gtk_style_attach (widget->style, widget->window);
  if (!led->gc)
    {
      cmap = gtk_widget_get_colormap (widget);
      
      if (!(&(led->fg[LED_COLOR_ON])))
	gdk_color_parse ("#00F100", &(led->fg[LED_COLOR_ON]));
      gdk_color_alloc (cmap, &(led->fg[LED_COLOR_ON]));
      if (!(&(led->fg[LED_COLOR_OFF])))
	gdk_color_parse ("#008C00", &(led->fg[LED_COLOR_OFF]));
      gdk_color_alloc (cmap, &(led->fg[LED_COLOR_OFF]));
      led->gc = gdk_gc_new (widget->window);
      gdk_gc_copy (led->gc, widget->style->white_gc);
    }
	

}

static gint
led_lit (int pos, int val) {
	static int lookup[10]= {125,80,55,87,90,79,111,81,127,95};
	return ((1<<pos)&lookup[val]);
}

static void
gtk_led_draw(GtkWidget *widget, GdkRectangle *area) {
  GtkLed   *led;
  GtkMisc  *misc;
  GdkColor *win_bg;
  gint     i;
  gint     w, val, start_x, start_y;

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_LED (widget));
  g_return_if_fail (area != NULL);
  
  led = GTK_LED (widget);
  misc = GTK_MISC (widget);

	if (GTK_WIDGET_DRAWABLE (widget)) {
		if ((widget->allocation.width >= widget->requisition.width) &&
		    (widget->allocation.height >= widget->requisition.height)) {
			guint x, y;
			win_bg = &(led->fg[LED_COLOR_OFF]);
			gdk_gc_set_foreground (led->gc, win_bg);
			x = widget->allocation.x;
			y = widget->allocation.y;

//fprintf(stderr, "gtkled: height = %d, width = %d\n", widget->allocation.height, widget->allocation.width);
			if(widget->allocation.height < 5)
				return;

			gdk_draw_rectangle (widget->window,
			  led->gc,
			  TRUE,
			  x + 1, y + 1,
			  widget->allocation.width - 2,
			  widget->allocation.height - 4);

			val = led->value;
			start_x = x+3;
			start_y = y+3;
			x = widget->allocation.width - 6;
			y = widget->allocation.height/2-5;
			w = MIN(x,y) /6;
			if (w<3) w=3;
			if (!(w%2)) w++;

			for (i=0;i<w;i++) {
				if (led_lit(0,val)) gdk_gc_set_foreground(led->gc, &(led->fg[LED_COLOR_ON]));
				else gdk_gc_set_foreground(led->gc, &(led->fg[LED_COLOR_OFF]));
				gdk_draw_line (widget->window, led->gc,
				           start_x+1+i, start_y+i, start_x+x-2-i, start_y+i);
				if (led_lit(1,val)) gdk_gc_set_foreground(led->gc, &(led->fg[LED_COLOR_ON]));
				else gdk_gc_set_foreground(led->gc, &(led->fg[LED_COLOR_OFF]));
				if (!(i%2)) gdk_draw_line (widget->window, led->gc,
				                       start_x+2+(i/2),   start_y+y-(i/2)+1,
				                       start_x+x-3-(i/2), start_y+y-(i/2)+1);
				else     gdk_draw_line (widget->window, led->gc,
				                    start_x+2+(i/2)+1,       start_y+y+(i/2)+2,
				                    start_x+x-3-((i/2)+1), start_y+y+(i/2)+2);
				if (led_lit(2,val)) gdk_gc_set_foreground(led->gc, &(led->fg[LED_COLOR_ON]));
				else gdk_gc_set_foreground(led->gc, &(led->fg[LED_COLOR_OFF]));
				gdk_draw_line (widget->window, led->gc,
				           start_x+1+i,   start_y+2*y-i+2,
				           start_x+x-2-i, start_y+2*y-i+2);
				if (led_lit(3,val)) gdk_gc_set_foreground(led->gc, &(led->fg[LED_COLOR_ON]));
				else gdk_gc_set_foreground(led->gc, &(led->fg[LED_COLOR_OFF]));
				gdk_draw_line (widget->window, led->gc,
				           start_x+i, start_y+2+i, start_x+i, start_y+y-i);
				if (led_lit(4,val)) gdk_gc_set_foreground(led->gc, &(led->fg[LED_COLOR_ON]));
				else gdk_gc_set_foreground(led->gc, &(led->fg[LED_COLOR_OFF]));
				gdk_draw_line (widget->window, led->gc,
				           start_x+x-i-1, start_y+2+i, start_x+x-i-1, start_y+y-i);
				if (led_lit(5,val)) gdk_gc_set_foreground(led->gc, &(led->fg[LED_COLOR_ON]));
				else gdk_gc_set_foreground(led->gc, &(led->fg[LED_COLOR_OFF]));
				gdk_draw_line (widget->window, led->gc,
				           start_x+i, start_y+2+i+y, start_x+i, start_y+2*y-i);
				if (led_lit(6,val)) gdk_gc_set_foreground(led->gc, &(led->fg[LED_COLOR_ON]));
				else gdk_gc_set_foreground(led->gc, &(led->fg[LED_COLOR_OFF]));
				gdk_draw_line (widget->window, led->gc,
				           start_x+x-i-1, start_y+2+i+y, start_x+x-1-i, start_y+2*y-i);
			}
		}
	}
}
/* EOF */

